****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createclaimsfiles.sas  
*
* Created (mm/dd/yyyy): 05/15/2018
* Last modified: 10/25/2018
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
* 	Extract and save claims for identified cohort for HDPS/MFU                                     
*   
*  Program inputs:   
*
*                                     
*  Program outputs:  
*	-Claims_icddx09.sas7bdat
*   -Claims_icddx10.sas7bdat   
*	-Claims_dxOT.sas7bdat		 
*   -Claims_icdpx09.sas7bdat   
*	-Claims_icdpx10.sas7bdat   
*	-Claims_pxOT.sas7bdat		 
*   -Claims_cpt.sas7bdat       
*   -Claims_hcpcs.sas7bdat     
*   -Claims_drugclass.sas7bdat 
*	-Claims_rx.sas7bdat		 
*	-Claims_lab.sas7bdat		 
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1	      10/25/18		AP		Added MI HDPS capability
*
***************************************************************************************************;

%macro ms_createclaimsfiles;

	%put =====> MACRO CALLED: ms_createclaimsfiles v1.1;

	%let HDPS=N;
	%let MFU=N;

    %ISDATA(dataset=HDPSSettings);
	*Note: HDPS settings contains groups where HDPS=Y;	
    %IF %EVAL(&NOBS.>0) %THEN %DO;
    	data _ToSelect1;
        	set HDPSSettings;
            where upcase(HDPS)="Y";
            if upcase(HDPS)="Y" then call symputx("HDPS",upcase(HDPS));
            keep group;
        run;
        %put &HDPS.;
    %END;
	%else %do;
		data _ToSelect1;
			format group $40.;
			group = '';
			stop;
		run;
	%end;

	*Note: MFU is performed on all groups, so _ToSelect contains all groups, even if HDPS is not performed on all groups;
	%isdata(dataset=infolder.&mfufile.);
	%if %eval(&nobs.>0) %then %do;
		data _ToSelect2(rename=cohortgrp=group);
			set cohortfile;
			keep cohortgrp;
		run;
	%end;
	%isdata(dataset=_ToSelect2);
	%if %eval(&nobs.>0) %then %do;
		%let MFU=Y;
		%put &MFU;
	%end;
	%else %do;
		data _ToSelect2;
			format group $40.;
			group = '';
			stop;
		run;
	%end;

	data _ToSelect;
		set _ToSelect1 _ToSelect2;
	run;

    %if %str(&HDPS.)=%str(Y) | %str(&MFU.)=%str(Y) %then %do; 

		*complete list of groups;
		data _claimsdata;
			set %if %str(&MFU.)=%str(Y) | %eval(&type.=2) %then %do; &runid._mstr&preg. %end;
				%if %eval(&milinkage. =1) %then %do;
					%if %str(&HDPS.)=%str(Y) %then %do;
					&runid._mstr_mi
					%end;
				%end;
				;
			keep Patid group Indexdt;
		run;

    	*keep only patients in groups where HDPS is required (All groups for MFU);
        data _lookup(keep=Patid group Indexdt);
        	if 0 then set _ToSelect(Keep=group);
                declare hash ht (hashexp:16, dataset:"_ToSelect");
                ht.definekey("group");
                ht.definedone();

                do until(eof1);
                  set _claimsdata end=eof1;
                  if ht.find()=0 then output;
                end;
            stop;
		run;

		*keep only new patients;
		%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
			proc sort nodupkey data=_NewPatients(keep=group patid);
				by group patid;
			run;

			data _lookup;
				merge _lookup(in=a)
				      _NewPatients(in=b);
				by group PatId;
				if a and b;
			run;
		%end;

		/*Determine Min and Max Windows. Cannot use coalesce() because HDPS or MFU may use entire history 
		  whereas the other is a defined window */
			%let HDPS_MFU_FROM=.;
			%let HDPS_MFU_TO=.;

			/*Enrollment considered absolute value of window, need to obtain true max window*/
			%if %str(&MFU.)=%str(Y) %then %do;
				proc means data=infolder.&mfufile. nway noprint;
				    var MFUFROM MFUTO;
				    output out=MFU_WIN min(MFUFROM)= 
				                        max(MFUTO)=;
			    run;
			    
			    data _NULL_;
				    set MFU_WIN;
				    call symputx("MinMFUWinFrom",put(MFUFROM,best.));
				    call symputx("MaxMFUWinTo",put(MFUTO,best.));
			    run;

				%put &MinMFUWinFrom. &MaxMFUWinTo.;
			%end;
		    %put &MinHDPSWinfrom. &MaxHDPSWinTo.;

			%if %str(&HDPS.)=%str(Y) & %str(&MFU.)=%str(N) %then %do; 
				%let HDPS_MFU_FROM=&MinHDPSWinfrom.;
				%let HDPS_MFU_TO=&MaxHDPSWinTo.;
			%end;
			%else %if %str(&HDPS.)=%str(N) & %str(&MFU.)=%str(Y) %then %do; 
				%let HDPS_MFU_FROM=&MinMFUWinFrom.;
				%let HDPS_MFU_TO=&MaxMFUWinTo.;
			%end;
			%else %do; /*both HDPS and MFU*/
				%if &MinHDPSWinfrom. ne . & &MinMFUWinFrom. ne . %then %let HDPS_MFU_FROM = %sysfunc(min(&MinHDPSWinfrom.,&MinMFUWinFrom.));
				%else %let HDPS_MFU_FROM = .;

				%if &MaxHDPSWinTo. ne . & &MaxMFUWinTo. ne . %then %let HDPS_MFU_TO = %sysfunc(max(&MaxHDPSWinTo.,&MaxMFUWinTo.));
				%else %let HDPS_MFU_TO = .;
			%end;

			%let t3vars = ;
			%if %eval(&type.=3) %then %do;
				%let t3vars = , pts.analysiscohort, pts.censordeath, pts.censorenrol;
			%end;

            *Claims_icddx;
            proc sql noprint;
                create table &RUNID._Claims_icddx as
                select /*distinct*/ claims.Patid,
                                compress(claims.DX) as Code,
								claims.DX_Codetype,
                                claims.Adate,
								claims.enctype,
								claims.pdx,
                                pts.group,
								pts.indexdt
								&t3vars.
                from _lookup as pts, 
                     &DataLib.&Diatable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&HDPS_MFU_FROM.,-99999) pts.IndexDt+coalesce(&HDPS_MFU_TO.,99999),                                             
                                         period2=claims.ADate)
                order by claims.Patid, 
                         pts.group;
			quit;

  			data DPLocal.&RUNID._Claims_icddx09
                 DPLocal.&RUNID._Claims_icddx10
				 DPLocal.&RUNID._Claims_dxOT;
                set &RUNID._Claims_icddx; 

                if DX_Codetype in("09") then output DPLocal.&RUNID._Claims_icddx09;
				if DX_Codetype in("10","11") then output DPLocal.&RUNID._Claims_icddx10;
				if DX_Codetype in("OT") then output DPLocal.&RUNID._Claims_dxOT; /*For MFU*/
			run;

            *Claims_icdpx;
            proc sql noprint;
                create table &RUNID._Claims_icdpx as
                select /*distinct*/ claims.Patid,
                                compress(claims.PX) as Code,
                                pts.group,
                                claims.PX_Codetype,
                                claims.Adate,
								claims.Enctype,
								pts.indexdt
								&t3vars.
                from _lookup as pts, 
                     &DataLib.&Proctable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&HDPS_MFU_FROM.,-99999) pts.IndexDt+coalesce(&HDPS_MFU_TO.,99999),                                             
                                         period2=claims.ADate)
                order by claims.Patid, 
                         pts.group;
			quit;

            data DPLocal.&RUNID._Claims_icdpx09
				 DPLocal.&RUNID._Claims_icdpx10
                 DPLocal.&RUNID._Claims_cpt
                 DPLocal.&RUNID._Claims_hcpcs
				 DPLocal.&RUNID._Claims_pxOT;
                set &RUNID._Claims_icdpx; 

                if PX_Codetype in("09") then output DPLocal.&RUNID._Claims_icdpx09;
                if PX_Codetype in("10","11") then output DPLocal.&RUNID._Claims_icdpx10;
                if PX_Codetype in("C2","C3","C4") then output DPLocal.&RUNID._Claims_cpt;
                if PX_Codetype in("HC","H3")      then output DPLocal.&RUNID._Claims_hcpcs;
				if PX_Codetype in("RE","ND","LO","OT") then output DPLocal.&RUNID._Claims_pxOT; /*For MFU*/
            run;

            *Claims_drugclass;
            proc sql noprint;
            	create table _Claims_drugclass as
                select /*distinct*/ claims.Patid,
                                claims.NDC as Code,
                                claims.Rxdate as Adate,
                                pts.group,
								pts.indexdt,
								claims.rxsup,
								claims.rxamt
								&t3vars.
                from _lookup as pts, 
                     &DataLib.&distable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&HDPS_MFU_FROM.,-99999) pts.IndexDt+coalesce(&HDPS_MFU_TO.,99999),                                             
                                         period2=claims.RxDate) /*claims.ADate+RxSup-1*/;
			quit;

			/*For MFU*/
			data DPLocal.&RUNID._Claims_rx;
				set _Claims_drugclass;
				where rxsup >0 and rxamt > 0;
			run;

            *add Drugclass information for HDPS;
            proc sql noprint;
                create table DPLocal.&RUNID._Claims_drugclass as
                select /*distinct*/ claims.Patid,
									claims.indexdt,
                                claims.group,
                                claims.adate,
                                ndc.classname as Code
                from infolder.&DRUGCLASSFILE. as ndc
                     inner join 
                     _Claims_drugclass as claims
                on ndc.ndc = claims.Code
                order by claims.Patid, 
                         claims.group;
            quit;

			/*Lab table - for MFU only*/
			%ISDATA(dataset=indata.&labtable.);	
   			%IF %EVAL(&NOBS.>0) %THEN %DO;
		   		proc sql noprint;
		       		create table DPLocal.&RUNID._Claims_lab as
		       		select claims.patid,
						   pts.group,
						   pts.indexdt,
						   coalesce(claims.lab_dt,claims.result_dt,claims.order_dt,0) as adate, 
		       		       upcase(claims.result_type) as result_type, 
		                   claims.loinc,
		       		       claims.px, 
		                   claims.px_codetype,
						   claims.ms_test_name,
						   claims.specimen_source,
						   claims.ms_test_sub_category,
						   claims.ms_result_unit,
						   claims.fast_ind,
						   claims.pt_loc
						   &t3vars.
		       		from _lookup as pts, 
					     &DataLib.&labtable. as claims
					where pts.Patid = claims.Patid and %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&HDPS_MFU_FROM.,-99999) pts.IndexDt+coalesce(&HDPS_MFU_TO.,99999),                                             
					                                      period2=coalesce(claims.lab_dt,claims.result_dt,claims.order_dt,0))
					order by claims.Patid, pts.group;
				quit;
			%end; *Lab MFU codes;
				
			%if ("&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p") & %eval(&PERIODIDSTART.>=2)  %then %do;

			  data DPLocal.&RUNID._Claims_icdpx09;
				  set  DPLocal.&RUNID._Claims_icdpx09
				      &DPLOCALPOINTER..&RUNID._Claims_icdpx09;
			  run;

			  data DPLocal.&RUNID._Claims_icdpx10;
				  set  DPLocal.&RUNID._Claims_icdpx10
				      &DPLOCALPOINTER..&RUNID._Claims_icdpx10;
			  run;

			  data DPLocal.&RUNID._Claims_cpt;
				  set  DPLocal.&RUNID._Claims_cpt
				      &DPLOCALPOINTER..&RUNID._Claims_cpt;
			  run;

			  data DPLocal.&RUNID._Claims_hcpcs;
				  set  DPLocal.&RUNID._Claims_hcpcs
				      &DPLOCALPOINTER..&RUNID._Claims_hcpcs;
			  run;

			  data DPLocal.&RUNID._Claims_pxOT;
				  	set DPLocal.&RUNID._Claims_pxOT
				       &DPLOCALPOINTER..&RUNID._Claims_pxOT;
			  run;

			  data DPLocal.&RUNID._Claims_icddx09;
				  set  DPLocal.&RUNID._Claims_icddx09
				      &DPLOCALPOINTER..&RUNID._Claims_icddx09;
			  run;

			  data DPLocal.&RUNID._Claims_icddx10;
				  set  DPLocal.&RUNID._Claims_icddx10
				      &DPLOCALPOINTER..&RUNID._Claims_icddx10;
			  run;

			  data DPLocal.&RUNID._Claims_dxOT;
				  set DPLocal.&RUNID._Claims_dxOT
				     &DPLOCALPOINTER..&RUNID._Claims_dxOT;
			  run;

			  data DPLocal.&RUNID._Claims_drugclass;
				  set  DPLocal.&RUNID._Claims_drugclass
				      &DPLOCALPOINTER..&RUNID._Claims_drugclass;
			  run;

			  data DPLocal.&RUNID._Claims_rx;
				  set  DPLocal.&RUNID._Claims_rx
				      &DPLOCALPOINTER..&RUNID._Claims_rx;
			  run;

			 %ISDATA(dataset=indata.&labtable.);	
   		     %IF %EVAL(&NOBS.>0) %THEN %DO;
				data DPLocal.&RUNID._Claims_lab;
				  set  DPLocal.&RUNID._Claims_lab
				      &DPLOCALPOINTER..&RUNID._Claims_lab;
				run;
			%end;
		%end;
	%end;  *HDPS/MFU extraction;

	%put NOTE: ******** END OF MACRO: ms_createclaimsfiles  v1.1 ********;

%mend ms_createclaimsfiles;
