****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_extractmeds.sas  
*
* Created (mm/dd/yyyy): 07/31/2014
* Last modified: 11/20/2014
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program pre extracts raw diagnosis or procedure claims based solely on code match. This is 
*  	useful to use as a pre-extraction to avoid claim duplication;
*
*  Program inputs:                                                                                   
*  	-dataset to extract records from
* 	-dataset containing the list of codes to extract
*
*  Program outputs:                                                                                                                                       
*  	-dataset containing the extracted records
*             
* PARAMETERS:
*  	-datafile		= Name of the dataset to extract records from.
*  	-datacode		= Name of the variable containing the codes.
*  	-DataCodeType	= Name of the variable containing the CodeType.
*  	-lookfile		= Name of the sas dataset containing the list of codes to extract.  The name of the variable 
*           		  containing the codes must be called "code" and the one containing the codetype must be
*           		  called "codetype".
*  	-outfile		= Name of the output dataset containing the extracted records.
*  	-ExtraVars		= To add extra variables to initial proc sort nodupkey.
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1	  11/12/18	  AP		Converted proc sql to hash table to improve processing efficiency
*
***************************************************************************************************;

%MACRO ms_extractmeds(datafile=, datacode= , DataCodeType=, lookfile=, outfile=, ExtraVars=);

%put =====> MACRO CALLED: ms_extractmeds v1.1;

    %ISDATA(dataset=&lookfile.);

	%IF %EVAL(&NOBS.>=1) %THEN %DO;
  
        proc sort nodupkey data=&lookfile. 
                           out=_list;
        by code codetype &ExtraVars.;
        run;

		data &outfile.;
			if 0 then set _list;
			dcl hash h(dataset:'_list');
			h.definekey('code','codetype');
			h.definedata(all:'yes');
			h.definedone();

			do until (eof);
				set &datafile. end=eof;
				code=compress(&datacode.,'.');
				codetype=&DataCodeType.;
				if h.find()=0 then output;
			end;
		run;

    %END;
    %ELSE %DO;
        data &outfile.;
        set &datafile.(obs=0) &lookfile.(obs=0);
        run;
    %END;

    proc datasets library=work nolist nowarn;
    delete _list;
    quit;

    %put NOTE: ********END OF MACRO: ms_extractmeds v1.1********;

%MEND ms_extractmeds;       


