****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_reportutilitymacros.sas  
*
* Created (mm/dd/yyyy): 11/12/2017
* Last modified: 12/18/2018
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The file holds several utility macros for reporting tools
*	- %rearrangevars
*	- %tableletter
*	- %isblank
*	- %create_comma_charlist
*   - %nonrep
*   - %convert_strat
*   - %add_comma_charlist
*
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  12/18/18	  AP		Added Type 5 report utility macros
*
***************************************************************************************************;

*Macro for recoding variables for output;
%macro rearrangevars(dataset=);

	data &dataset.;
		set &dataset.;
		%if %index(%lowcase(&category.), month) %then %do;
			length _month $10; label _month = "Month";
			_month = put(month, mn_name.);
			drop month;
			rename _month = month;
		%end;

		/*ageformat*/
		%if %index(%lowcase(&category.), agegroup) & %length(&agegroupfmt.) > 0 %then %do;
			length _agegroup $30; label _agegroup = "Age Group";
			_agegroup = put(agegroup, $agefmt.);
			drop agegroup;
			rename _agegroup = agegroup;
		%end;

		/*covariate*/
		%if %index(%lowcase(&category.),covar) >0 %then %do;
			%do c = 1 %to %sysfunc(countw(&category.));
				%let w = %lowcase(%scan(&category., &c.));
				%if %index(&w., covar)> 0 %then %do;
					%let covnum = %substr(&w., 6);
					length _covar&covnum. $150;
					label _covar&covnum. = "&&&covar&covnum.";
					if covar&covnum. = 0 then _covar&covnum. = "No evidence of &&&covar&covnum.";
					if covar&covnum. = 1 then _covar&covnum. = "Evidence of &&&covar&covnum.";
				drop covar&covnum.;
				rename _covar&covnum. = covar&covnum.;
				%end;
			%end;
		%end;

		*HHS format;
		%if %index(%lowcase(&category.), hhs_reg) > 0 %then %do;
			length _hhs_reg $30; label _hhs_reg = "HHS Region";
			_hhs_reg = put(HHS_reg, $hhs_regionfMT.);
			drop hhs_reg;
			rename _hhs_reg = hhs_reg;
		%end;
	run;

	%if %index(%lowcase(&category.), year) > 0 %then %do;
		proc sort data = &dataset.;
			by year;
		run;
	%end;
	%if %index(%lowcase(&category.), cb_reg) > 0 %then %do;
		data &dataset.;
			set &dataset.;
			cb_order = put(cb_reg, $census_order.);
		run;
		proc sort data = &dataset. out=&dataset.(drop=cb_order);
			by cb_order;
		run;
	%end;
	

	data &dataset.;
		set &dataset.;
			length newcategory $80;
			newcategory = "";
		 %if %sysfunc(countw(%quote(&category.))) >=2 %then %do;
			%do cat = 1 %to %eval(%sysfunc(countw(%quote(&category.)))-1);
				%if &cat. = 1 %then %do;
					%if %index(%lowcase(%scan(%quote(&category.), &cat.)), agegroup) %then 
						newcategory = "Aged "||strip(%scan(%quote(&category.), &cat.));
					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), zip3) %then 
						newcategory = "3-Digit Zip/State ("||strip(%scan(%quote(&category.), &cat.))||")";
/*					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), hhs_reg) %then */
/*						newcategory = "HHS Region ("||strip(%scan(%quote(&category.), &cat.))||")";*/
					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), cb_reg) %then 
						newcategory = "Census Region ("||strip(%scan(%quote(&category.), &cat.))||")";
					 %else
						newcategory = strip(%scan(%quote(&category.), &cat.));;
				%end;
				%else %do;
					%if %index(%lowcase(%scan(%quote(&category.), &cat.)), agegroup) %then 
					newcategory = cat(strip(newcategory),", ", "Aged "||strip(%scan(%quote(&category.), &cat.)));
					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), zip3) %then 
					newcategory = cat(strip(newcategory),", ", "3-Digit Zip/State ("||strip(%scan(%quote(&category.), &cat.)),")");
/*					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), hhs_reg) %then */
/*					newcategory = cat(strip(newcategory),", ", "HHS Region ("||strip(%scan(%quote(&category.), &cat.)),")");*/
					%else %if %index(%lowcase(%scan(%quote(&category.), &cat.)), cb_reg) %then 
					newcategory = cat(strip(newcategory),", ", "Census Region ("||strip(%scan(%quote(&category.), &cat.)),")");
					%else 
					newcategory = cat(strip(newcategory),", ", strip(%scan(%quote(&category.), &cat.)));;
				%end;
			%end;				 
		%end;
	run;
%mend rearrangevars;


*Macro for incrementing table letters;
%macro tableletter();
	%let div = %sysfunc(int(&tablecount. / 27)); %put &div.;
	%let secondplace = %eval(&tablecount - (26*&div));%put &secondplace.;

	%if %eval(&div = 0) %then %do;
		%let tableletter = %scan(a b c d e f g h i j k l m n o p q r s t u v w x y z, &tablecount.);
	%end;
	%else %do;
		%let tableletter = %scan(a b c d e f g h i j k l m n o p q r s t u v w x y z, &div.)%scan(a b c d e f g h i j k l m n o p q r s t u v w x y z, &secondplace);
	%end;

	%let tablecount = %eval(&tablecount + 1); /*+1 to counter*/
%mend;


*Macro for determining whether macro variable is blank;
%macro isBlank(param);
	 %sysevalf(%superq(param)=,boolean)
%mend isBlank;


*Macro for converting macro variable with space deliminated list to comma deliminated list with quotation around each word;
%macro create_comma_charlist(inlist=, outlist=);
	%global &outlist.;

	%let countvars = %sysfunc(Countw(%quote(&inlist.)));
		%do c = 1 %to &countvars.;
			%let word = %scan(%quote(&inlist),&c.);
				%if &c. = 1 %then %do;
					%let &outlist. = "&word.";
				%end;
				%else %do;
					%let &outlist. = &&&outlist. , "&word.";
				%end;
		%end;
	%let &outlist = %upcase(&&&outlist);
%mend create_comma_charlist;

*Removes repeated words in a macro variable;
%macro nonrep(invar= , outvar= );
	%global &outvar;
	%let long = ;
	%if %str(&&&invar) ne %str() %then %do;
	%do w=1 %to %sysfunc(countw(&&&invar));
		%if %sysfunc(indexw(&long, %scan(&&&invar,&w))) = 0 %then %do;
			%let long = &long %scan(&&&invar,&w);
		%end;
	%end;
	%end;
	%let &outvar = &long.;
%mend;

*Converts categories into executable expression;
%macro convert_strat(var=, categories=);

	%global num_categories categories_boolean ;
	%let num_categories = ;
	%let categories_boolean = ;

	%let num_categories = %sysfunc(countw(&categories, ' '));
	%do loop = 1 %to &num_categories;
		/*variable list*/
		/*mathematical expressions*/
		%let cat = %scan(&categories., &loop., ' ');
		%put &cat.;
		/*no range of values (i.e. <10, <=20, >60 >=75, 65+)*/
		%if %index(&cat., -) = 0 %then %do;
			%if %index(&cat.,+) > 0 %then %do;
				%let cat = &var>=%sysfunc(tranwrd(&cat., +, ));
			%end;
			%else %if (%index(&cat.,<) > 0 | %index(&cat.,<) > 0) %then %do;
				%let cat = &var.&cat.;
			%end;
			%else %do;
				%let cat = &var.=&cat.;
			%end;
		%end;
		/*range of values inclusive of boundry (i.e. 25-50)*/
		%else %if %index(&cat., -) > 0 & %index(&cat., <) = 0 & %index(&cat., >) = 0 %then %do;
			%let cat = %sysfunc(tranwrd(&cat., -, <=&var.<=));
		%end;
		/*range of values inclusive of upper boundry, (i.e. 25<-60)*/
		%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 1, '-'), <) > 0 %then %do;
			%let cat = %scan(&cat., 1, '-')&var.<=%scan(&cat., 2, '-');
		%end;
		/*range of values inclusive of lower bountry, (i.e. 50-<75)*/
		%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 2, '-'), <) > 0 %then %do;
			%let cat = %scan(&cat., 1, '-')<=&var.%scan(&cat., 2, '-');
		%end;

		%let categories_boolean = &categories_boolean. &cat.;
	%end;

	%put &categories &categories_boolean.;
%mend;

/*Creates comma separated list*/
%macro add_comma_charlist(inlist=, outlist=);
	%global &outlist.;

	%let countvars = %sysfunc(Countw(%quote(&inlist.)));
		%do ccc = 1 %to &countvars.;
			%let word = %scan(%quote(&inlist),&ccc.);
				%if &ccc. = 1 %then %do;
					%let &outlist. = &word.;
				%end;
				%else %do;
					%let &outlist. = &&&outlist. , &word.;
				%end;
		%end;
	%let &outlist = %upcase(&&&outlist);
%mend add_comma_charlist;
