options linesize=100 pagesize=50;
/****************************************************************************************
*                                     SENTINEL PROGRAM
*****************************************************************************************
* NAME:  
*
* PURPOSE:
*
* MAJOR STEPS: 
*
* KEY DEPENDENCIES/CONSTRAINTS/CAVEATS:
*
*----------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*----------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yyyy):
*  Last modified date (mm/dd/yyyy): 
*  Version: 
*
****************************************************************************************/

/*---------------------------------------------------------------------------------------
*                                      DEFINITIONS
*----------------------------------------------------------------------------------------
* SCC... Location of Sentinel Common-Components (CC) request associated with the most
*        recent production ETL for either Phase A (standard SCDM) or Phase B (standard
*        SCDM plus MIL table). Specific to each Data Partner/Data Site, the CC utility 
*        macro defines key:
*          - macro variables (e.g. Data Partner ID, Min and Max Dates, SCDM table names) 
*          - libname assignments (e.g. libref INDATA is assigned to SCDM tables)
*        NOTE: This parameter needs to be set by programmer at the Data Partner/Data Site
*
* ReqID... Sentinel Project Request-ID (formerly MSReqID)
*          NOTE: This parameter needs to be set by the programmer preparing the 
*               Request-ID in coordination with the Sentinel Operations Center (SOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e. behind the DP firewall)
*    MSOC     ... path for data created to share with Sentinel Operations Center
*    INFOLDER ... path for program inputs (e.g. lookup data, program modules, macros)
*    SASPROGRAMS ... path for the main SAS program for the request
*--------------------------------------------------------------------------------------*/

*---------------------------------------------------------------------------------------
* NOTES:
*  For reasons of compatibility and standardization, ALL paths must meet the following
*  criteria:
*     1- Starting with the request directory, all subdirectory names must be specified
*        in lower case (for UNIX compatibility)
*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
*     4- Paths must not contain any embedded spaces
*---------------------------------------------------------------------------------------;
/*-------------------------------------------------------------------------------------*/
/* Section 1: Data Partner/Data Site programmer defines parameters in this section.     */
/*-------------------------------------------------------------------------------------*/
/* 1a. Edit macro variable SCC to point to the directory containing the executed Common
         Components (CC) request associated with the relevant production ETL, INCLUDING 
         the Request-id. 
       NOTE: ORGANIZATIONS WITHOUT CC SHOULD LEAVE BLANK AND COMMPLETE ALL OF SECTION 1c.             
       Example: %let SCC = /sentinel/requests/etl4/soc_cca_wp001_xxxx_v01/             */
   %let SCC = ;

/* 1b. Program-specific parameters                                                     */

	/* Edit this parameter to identify the list of patients to exclude (if applicable).
	   Dataset should contain one variable, patid, and list all patids that should be
	   excluded. Must include libname (e.g., indata.PtsToExclude).
	 */
	%let PTSTOEXCLUDE=;

	/* Edit this parameter to make it point to the folder containing dplocal datasets
	   from the prior runs (Prior DPLocal) 
	*/
	%let DPLPrior=;   * Enter the path to the prior run data; 

/***************************************************************************************/
/* 1c. OPTIONAL: Organizations WITHOUT Common Components define parameters in this 
       section, leaving STEP 1a. above blank.                                          */

    /* _DP is a descriptive identifier for your organization. Specify a 3-6 UPPERCASE 
       character abbreviation for _DP. Example: %let DP=ABCDE ;                        */
       %let _DP= synpuf;

    /* Specify the location of this request package, containing the 4 subfolders: 
       dplocal, msoc, inputfiles, and sasprograms */  
    /* Example: %let packageroot = A:/sentinel/qa_mil_package/                         */
       %let _packageroot = C:\Public_Training_Materials\Lab\public_mpl1r_wp001_synpufs_v02;

    /* Specify the file path to the location of your Phase A Sentinel Common Data Model 
      (SCDM) datasets.  Example: %let prod_scdm=A:/sentinel/etl1/phaseA/scdm/ ;        */
       %let prod_scdm= C:\Public_Training_Materials\Lab\test_data;

    /* The following metadata describe characteristics of your SCDM. If known, modify the
       following parameters:      
    /* DP_MinDate: Specify the overall SCDM minimum date in the format "DDMMMYYYY"d 
                   Default value is "01Jan2000"d                                       */
       %let _DP_MinDate="01Jan2008"d ;

    /* DP_MaxDate: Specify the overall SCDM maximum date in the format "DDMMMYYYY"d 
                    Default value is today's date (e.g. "&sysdate."d)                  */
       %let _DP_MaxDate = "01Dec2010"d ;

    /* ETL: Please enter the ETL version # for the production SCDM. 
              Default is 1                                                             */
       %let _ETL = 1;

    /* Phase: Please enter the Phase (A or B) of the associated ETL SCDM. 
              Default is A                                                             */
       %let _Phase = A;

    /* Specify the name of each of the Sentinel Common Data Model tables, if different
       from the default values entered. Do NOT include '.sas7bdat'. DO NOT leave blank.
       Example: %let _ENRTABLE=enrollment ;                                            */
       %let _ENRTABLE=  enrollment_sp1;  *specify the enrollment table name *;
       %let _DEMTABLE=  demographic_sp1; *specify the demographic table name *;
       %let _DISTABLE=  dispensing_sp1;  *specify the rx dispensing table name *;
       %let _ENCTABLE=  encounter_sp1;   *specify the encounter table name *;
       %let _DIATABLE=  diagnosis_sp1;   *specify the diagnosis table name *;
       %let _PROCTABLE= procedure_sp1;   *specify the procedure table name *;

    /****************************************************************************\
       The following tables are optional. Leave BLANK if they are not available 
       at your site (DO NOT COMMENT OUT).
    \****************************************************************************/
       %let _DEATHTABLE= death_sp1;  *specify the death table name *;
       %let _CODTABLE= ;    		 *specify the cause of death table name *; 
       %let _LABTABLE= ;    		 *specify the lab result table name *;
       %let _VITTABLE= ;    		 *specify the vital signs table name *;
       %let _IPHARMTABLE= ; 		 *specify the inpatient pharmacy table name *;
       %let _ITRANSTABLE= ; 		 *specify the inpatient transfusion table name *;
       %let _MILTABLE= ;    		 *specify the mother-infant-linkage table name (Phase B only);

/********************************** END OF SECTION 1 ***********************************/

/*-------------------------------------------------------------------------------------*/
/* SECTION 2: SOC Programmer/Analyst preparing the Request sets each parameter prior
              to distributing to DP 
/*-------------------------------------------------------------------------------------*/
 /* Specific request IDs are made up of the following 5 tokens:
    project-ID, workplan-type, workplan-ID, unique-DPID, version-ID */
 /*-------------------------------------------------------------------------------*/
  *  If this is your request   . . . . . . . then set parameter values as follows
  *  ---------------------------------       --------------------------------------
  *   [Project-ID: CDER]                      %let ProjID= cder ;  
  *   [Workplan-Type: ad hoc request]         %let WPType = ahr ; 
  *   [Workplan-ID: 5]                        %let WPID =   wp005 ; 
  *   [Unique-DPID: non-specific DP]          %let DPID =   nsdp ;     
  *   [Version-ID: beta 3]                    %let VerID =  b03 ;         
  *   [use underscores as delimiter]          %let dlm = _ ;                
  *
  *   The example values above would produce Request-ID --> cder_ahr_wp005_nsdp_b03 ;
 /*------------------------------------------------------------------------------------*/
 /* Specify project-ID, workplan-type, workplan-ID, workplan-type, dpid, version-ID    */
   %let ProjID = public ; 
   %let WPType = mpl1r ; 
   %let WPID =   wp001 ; 
   %let DPID =   synpufs ;
   %let VerID =  v02 ;
 
  /* Create request-id delimiter - Default is underscore (_)                           */
   %let dlm = _ ;  /* Do not edit */     

****************************************************************************************
*******                             END OF USER INPUT                             ******
*******                        DO NOT EDIT BELOW THIS LINE                        ******
******* (Consult with SOC or Sentinel team leader if you feel edits are required) ******
****************************************************************************************;
/*-------------------------------------------------------------------------------------*/
/* SECTION 3a: Standard SOC environment setup code -- DO NOT EDIT
/*-------------------------------------------------------------------------------------*/
/* System options */
options nosymbolgen nomlogic;
options ls=100 nocenter ;
options obs=MAX ;
options msglevel=i ;
options mprint mprintnest ;
options errorcheck=strict errors=0 ;
options merror serror ;
options dkricond=error dkrocond=error mergenoby=warn;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options fullstimer ;
options missing = .;
options validvarname = v7;

%global ccbypass reqid;

/* Request-ID (must be lower case) */
%let ReqID = %lowcase(&ProjID.&dlm.&WPType.&dlm.&WPID.&dlm.&DPID.&dlm.&VerID);

%macro scc_yn; /* Please do not edit */
  /*-----------------------------------------------------------------------------------*/
  /* If common components specified, define all file paths                             */
  /*-----------------------------------------------------------------------------------*/
  %if %str("&SCC.") ne ("") %then %do;
    %let ccbypass=N;
    /* Include Common Components */
    %inc "&SCC./inputfiles/cc_utility/cc_macvars.sas" /nosource2 ;
    %end;
  /*-----------------------------------------------------------------------------------*/
  /* If common components NOT specified, define all file paths and CC macro parameters */
  /*-----------------------------------------------------------------------------------*/
  %else %if %str("&SCC.") = ("") %then %do;
    %let ccbypass=Y;
  /*-----------------------------------------------------------------------------------*/
  /* Define all macro parameters as global - DO NOT EDIT                               */
    %global etl dp phase dp_mindate dp_maxdate dplocal msoc infolder sasprograms
            enrtable demtable distable enctable diatable proctable  
            deathtable codtable labtable vittable ipharmtable itranstable miltable;
  /*-----------------------------------------------------------------------------------*/
    /* Assign Macro variables*/
    %let dp=&_dp;
    %let etl=&_etl;
    %let dp_mindate=%sysfunc(putn(%superq(_dp_mindate),best12.));
    %let dp_maxdate=%sysfunc(putn(%superq(_dp_maxdate),best12.));
    %let enrtable= &_enrtable;
    %let demtable= &_demtable;
    %let distable= &_distable;
    %let enctable= &_enctable;
    %let diatable= &_diatable;
    %let proctable= &_proctable;
    %let deathtable= &_deathtable;
    %let codtable= &_codtable;
    %let labtable= &_labtable;
    %let vittable= &_vittable;
    %let ipharmtable= &_ipharmtable;
    %let itranstable= &_itranstable;
    %let miltable=&_miltable;
    
    /* Delete temporary macro variables */
    %symdel _dp _etl _dp_mindate _dp_maxdate _enrtable _demtable _distable _enctable _diatable _proctable;
    %symdel _deathtable _codtable _labtable _vittable _ipharmtable _itranstable _miltable;

    %macro soc_clean_paths(paths);
      %local j ln subpath temppath;  
      %if %length(%superq(paths)) eq 0 %then %do;
      %put The parameter path must be non-missing.; %abort cancel; %end;
      %let paths=%qsysfunc(translate(&paths,%str(/),%str(\)));
      %do j=1 %to %qsysfunc(countw(&paths.,%str( )));
      %let subpath=%qscan(&paths,&j,%str( )); %let ln=%length(&subpath);
      %if %qsubstr(&subpath,&ln,1) ne %str(/) %then %do; %let subpath=&subpath./; %end;
      %let d_exist = %soc_dirExist(&subpath) ;  
      %if &d_exist eq 0 %then %do; %put Path &subpath. does not exist; %abort cancel; %end; 
      %let temppath=&temppath. &subpath.; %end;
      %let paths=%qleft(&temppath);
      &paths   /* returns value to calling environment, like a function */
    %mend soc_clean_paths;
    %macro soc_dirExist(dir) ; 
      %if %length(&dir.) eq 0 %then %do; 
      %put The parameter dir must be non-missing.; %abort cancel; %end;
      %local rc fileref return; %let rc=%qsysfunc(filename(fileref,&dir.)); 
      %let return=%qsysfunc(fexist(&fileref.));  
      &return  /* returns value to calling enivornment, like a function */
      %let rc=%qsysfunc(filename(fileref));  
    %mend soc_dirExist;
    %macro soc_quotepath(list);
      %local j d_exist path_ct path subpath temppath; %let list_ct=%qsysfunc(countw(&list,%str( )));
      %do j=1 %to &list_ct.;
      %let subpath=%qscan(&list,&j,%str( )); %let d_exist = %soc_dirExist(&subpath);  
      %if &d_exist eq 0 %then %do; %put Path &subpath does not exist; %abort cancel; %end;
      %let subpath="&subpath."; %let temppath=&temppath. &subpath.; %end;
      %let list=%qleft(&temppath);
      &list /* returns value to calling enivornment, like a function */
    %mend soc_quotepath;
    %macro soc_lib(ref, paths, options=) ;
      %local libpaths; %if %length(&ref) eq 0 %then %do; %put libref is blank; %abort cancel; %end;
      %if %length(%superq(paths)) eq 0 %then %do ;
      %put SOC-NOTE: For libref &ref the path is blank, libname assignment skipped; %end;
      %let libpaths= %soc_quotepath(&paths) ;
      libname &ref. %unquote((&libpaths.)) &options.;
    %mend soc_lib;

   /* Define request specific subdirectories */
    %let DPLOCAL = %soc_clean_paths(&_packageroot./dplocal/) ;
    %let MSOC = %soc_clean_paths(&_packageroot./msoc/) ;         
    %let INFOLDER = %soc_clean_paths(&_packageroot./inputfiles/) ;
    %let SASPROGRAMS = %soc_clean_paths(&_packageroot./sasprograms) ;
    %let INDATA=%soc_clean_paths(&prod_scdm) ;

    %soc_lib(INDATA, &INDATA, options=%str(access=readonly))
  %end;
%mend scc_yn;
%scc_yn
  
/* Assign libnames */
/* %soc_lib(libref-name, path-name, options=%str(options-name=options-value)) */
%soc_lib(DPLOCAL,&DPLOCAL)
%soc_lib(MSOC,&MSOC)
%soc_lib(INFOLDER,&INFOLDER, options=%str(access=readonly))

/* Delete macro variable used as delimiter as no longer needed */
%symdel dlm ;

/* Create clean work environment */
proc datasets lib=work kill memtype=data nolist nodetails;
quit;

****************************************************************************************
*****                                BEGIN PROGRAM                                 *****
****************************************************************************************
***** NOTE: Programmer developing a packages writes the rest of the program below  *****
****************************************************************************************;
/*-------------------------------------------------------------------------------------*/
/* 1-                                                                                  */
/*-------------------------------------------------------------------------------------*/ 

/********** FOLDER CONTAINING INPUT SAS MACROS USED BY THE MAIN PROGRAM *************************/;
/*Macro Files Location*/          %let sasmacr=&INFOLDER./macros/;			
/*Report Macro Files Location*/   %let reportmacr=&INFOLDER./macros/reportmacros/;			


*Added for Routine Querying System;
**Include macro and program files ** ;  
%include "&sasmacr.combo.sas";
%include "&sasmacr.ms_agestrat.sas";
%include "&sasmacr.ms_attrition.sas";
%include "&sasmacr.ms_caresettingprincipal.sas";
%include "&sasmacr.ms_cci_elix.sas";
%include "&sasmacr.ms_cidacov.sas";
%include "&sasmacr.ms_cidadenom.sas";
%include "&sasmacr.ms_cidanum.sas";
%include "&sasmacr.ms_cidatables.sas";
%include "&sasmacr.ms_cidatablest3.sas";
%include "&sasmacr.ms_cidatablest4.sas";
%include "&sasmacr.ms_cidatablest5.sas";
%include "&sasmacr.ms_cidatablest6.sas";
%include "&sasmacr.ms_codedistribution.sas";
%include "&sasmacr.ms_create_enrollment_spans.sas";
%include "&sasmacr.ms_createcat.sas";
%include "&sasmacr.ms_createcensortable.sas";
%include "&sasmacr.ms_createclaimepi.sas";
%include "&sasmacr.ms_createclaimsfiles.sas";
%include "&sasmacr.ms_createdistbaselinetable.sas";
%include "&sasmacr.ms_createnopreggroup.sas";
%include "&sasmacr.ms_createpov1.sas";
%include "&sasmacr.ms_createpov1t4.sas";
%include "&sasmacr.ms_createpov2.sas";
%include "&sasmacr.ms_createpov3.sas";
%include "&sasmacr.ms_createpov4.sas";
%include "&sasmacr.ms_createpov4t4.sas";
%include "&sasmacr.ms_createpov56.sas";
%include "&sasmacr.ms_createt5episodes.sas";
%include "&sasmacr.ms_createptsmasterlist.sas";
%include "&sasmacr.ms_createriskset.sas";
%include "&sasmacr.ms_debug.sas";
%include "&sasmacr.ms_delpatients.sas";
%include "&sasmacr.ms_envelope.sas";
%include "&sasmacr.ms_episoderec.sas";
%include "&sasmacr.ms_episoderec2.sas";
%include "&sasmacr.ms_extractanniversarydate.sas";
%include "&sasmacr.ms_extractdeaths.sas";
%include "&sasmacr.ms_extractdrugs.sas";
%include "&sasmacr.ms_extractlabs.sas";
%include "&sasmacr.ms_extractmeds.sas";
%include "&sasmacr.ms_extractmilinkage.sas";
%include "&sasmacr.ms_finalizemoiepisodes.sas";
%include "&sasmacr.ms_finalizeptsmasterlist.sas";
%include "&sasmacr.ms_findgap.sas";
%include "&sasmacr.ms_freezedata.sas";
%include "&sasmacr.ms_initmacrovariables.sas";
%include "&sasmacr.ms_loopmeds.sas";
%include "&sasmacr.ms_macros.sas";
%include "&sasmacr.ms_mfu.sas";
%include "&sasmacr.ms_moimetrics.sas";
%include "&sasmacr.ms_periodsoverlap.sas";
%include "&sasmacr.ms_processinputfiles.sas";
%include "&sasmacr.ms_processwildcards.sas";
%include "&sasmacr.ms_setnumloopmacrovars.sas";
%include "&sasmacr.ms_shaveinside.sas";
%include "&sasmacr.ms_shaveoutside.sas";
%include "&sasmacr.ms_squaredtableshell.sas";
%include "&sasmacr.ms_stockpiling.sas";
%include "&sasmacr.ms_surveillance.sas";
%include "&sasmacr.ms_time.sas";
%include "&sasmacr.ms_util.sas";

%include "&sasmacr.ms_nvrexposed.sas";

%include "&sasmacr.ms_covariate_adjustment.sas";
%include "&sasmacr.ms_matchtables.sas";
%include "&sasmacr.ms_nearestneighbormatch.sas";
%include "&sasmacr.ms_appendfiles.sas";
%include "&sasmacr.ms_hdps.sas";
%include "&sasmacr.ms_runmodel.sas";

%include "&sasmacr.ms_switching.sas";
%include "&sasmacr.ms_evalsecondaryepi.sas";
%include "&sasmacr.ms_evalsecondarytables.sas";
%include "&sasmacr.ms_its.sas";
%include "&sasmacr.ms_concomitantuse.sas";
%include "&sasmacr.ms_concomitantusetables.sas";

%include "&sasmacr.ms_createmicohorts.sas";

**Include reporting macros ** ;  
%include "&reportmacr.ms_t1t2_addstatetozip3.sas";
%include "&reportmacr.ms_t1t2_assignformats.sas";
%include "&reportmacr.ms_t1t2_createcdf.sas";
%include "&reportmacr.ms_t1t2_createreport.sas";
%include "&reportmacr.ms_t1t2_createbaselinetable.sas";
%include "&reportmacr.ms_t1t2_definegroupsruns.sas";
%include "&reportmacr.ms_t1t2_initializemacrovariables.sas";
%include "&reportmacr.ms_t1t2_outputbaselinetable.sas";
%include "&reportmacr.ms_t1t2_outputfigures.sas";
%include "&reportmacr.ms_t1t2_outputreport.sas";
%include "&reportmacr.ms_t1t2_outputt1t2table.sas";
%include "&reportmacr.ms_t1t2table.sas";

%include "&reportmacr.ms_t5_aggregate_tables.sas";
%include "&reportmacr.ms_t5_create_censoring_table.sas";
%include "&reportmacr.ms_t5_create_distribution_tables.sas";
%include "&reportmacr.ms_t5_create_figures.sas";
%include "&reportmacr.ms_t5_create_gaps_table.sas";
%include "&reportmacr.ms_t5_create_toc.sas";
%include "&reportmacr.ms_t5_createreport.sas";
%include "&reportmacr.ms_t5_output_report.sas";
%include "&reportmacr.ms_t5_set_reportparameters.sas";

%include "&reportmacr.ms_compute_baselinetable.sas";
%include "&reportmacr.ms_reportutilitymacros.sas";

**Include run_programs;
%include "&infolder.run_programs.sas";

*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+;
* End QRP                                          ;
*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+;
